/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_PASSAGE_H
#define DPC_IMPL_TYPE_PASSAGE_H

#include <cfloat>
#include <cmath>
#include "dpc/impl_type_index.h"
#include "impl_type_uint32.h"
#include "dpc/impl_type_longitudinalinfo.h"
#include "impl_type_uint8.h"
#include "impl_type_float.h"
#include "dpc/impl_type_vector_uint32_t.h"
#include "impl_type_boolean.h"

namespace dpc {
struct Passage {
    ::dpc::Index passage_id;
    ::UInt32 corridor_id{ 0U };
    ::dpc::LongitudinalInfo longt_info;
    ::UInt8 action_type{ 0U };
    ::UInt8 feature_label{ 0U };
    ::Float duration_time{ 0.0F };
    ::dpc::vector_uint32_t interest_obj_id_list;
    ::Float v_ego{ 0.0F };
    ::UInt8 lc_cls{ 0U };
    ::Boolean lc_passagegenflg{ false };
    ::Boolean lc_ego_frt_obj_exist_flg{ false };
    ::UInt32 lc_ego_frt_obj_id{ 0U };
    ::Boolean lc_tar_frt_obj_exist_flg{ false };
    ::UInt32 lc_tar_frt_obj_id{ 0U };
    ::Boolean lc_tar_bck_obj_exist_flg{ false };
    ::UInt32 lc_tar_bck_obj_id{ 0U };
    ::UInt8 lcp_cls{ 0U };
    ::Boolean lcp_frt_passagegenflg{ false };
    ::Boolean lcp_bck_passagegenflg{ false };
    ::Boolean lcp_ego_frt_obj_exist_flg{ false };
    ::UInt32 lcp_ego_frt_obj_id{ 0U };
    ::Boolean lcp_ego_bck_obj_exist_flg{ false };
    ::UInt32 lcp_ego_bck_obj_id{ 0U };
    ::Boolean lcp_nr_obj_exist_flg{ false };
    ::UInt32 lcp_nr_obj_id{ 0U };
    ::Boolean lcp_nr_frt_obj_exist_flg{ false };
    ::UInt32 lcp_nr_frt_obj_id{ 0U };
    ::Boolean lcp_nr_bck_obj_exist_flg{ false };
    ::UInt32 lcp_nr_bck_obj_id{ 0U };

    static bool IsPlane()
    {
        return false;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(passage_id);
        fun(corridor_id);
        fun(longt_info);
        fun(action_type);
        fun(feature_label);
        fun(duration_time);
        fun(interest_obj_id_list);
        fun(v_ego);
        fun(lc_cls);
        fun(lc_passagegenflg);
        fun(lc_ego_frt_obj_exist_flg);
        fun(lc_ego_frt_obj_id);
        fun(lc_tar_frt_obj_exist_flg);
        fun(lc_tar_frt_obj_id);
        fun(lc_tar_bck_obj_exist_flg);
        fun(lc_tar_bck_obj_id);
        fun(lcp_cls);
        fun(lcp_frt_passagegenflg);
        fun(lcp_bck_passagegenflg);
        fun(lcp_ego_frt_obj_exist_flg);
        fun(lcp_ego_frt_obj_id);
        fun(lcp_ego_bck_obj_exist_flg);
        fun(lcp_ego_bck_obj_id);
        fun(lcp_nr_obj_exist_flg);
        fun(lcp_nr_obj_id);
        fun(lcp_nr_frt_obj_exist_flg);
        fun(lcp_nr_frt_obj_id);
        fun(lcp_nr_bck_obj_exist_flg);
        fun(lcp_nr_bck_obj_id);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(passage_id);
        fun(corridor_id);
        fun(longt_info);
        fun(action_type);
        fun(feature_label);
        fun(duration_time);
        fun(interest_obj_id_list);
        fun(v_ego);
        fun(lc_cls);
        fun(lc_passagegenflg);
        fun(lc_ego_frt_obj_exist_flg);
        fun(lc_ego_frt_obj_id);
        fun(lc_tar_frt_obj_exist_flg);
        fun(lc_tar_frt_obj_id);
        fun(lc_tar_bck_obj_exist_flg);
        fun(lc_tar_bck_obj_id);
        fun(lcp_cls);
        fun(lcp_frt_passagegenflg);
        fun(lcp_bck_passagegenflg);
        fun(lcp_ego_frt_obj_exist_flg);
        fun(lcp_ego_frt_obj_id);
        fun(lcp_ego_bck_obj_exist_flg);
        fun(lcp_ego_bck_obj_id);
        fun(lcp_nr_obj_exist_flg);
        fun(lcp_nr_obj_id);
        fun(lcp_nr_frt_obj_exist_flg);
        fun(lcp_nr_frt_obj_id);
        fun(lcp_nr_bck_obj_exist_flg);
        fun(lcp_nr_bck_obj_id);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("passage_id", passage_id);
        fun("corridor_id", corridor_id);
        fun("longt_info", longt_info);
        fun("action_type", action_type);
        fun("feature_label", feature_label);
        fun("duration_time", duration_time);
        fun("interest_obj_id_list", interest_obj_id_list);
        fun("v_ego", v_ego);
        fun("lc_cls", lc_cls);
        fun("lc_passagegenflg", lc_passagegenflg);
        fun("lc_ego_frt_obj_exist_flg", lc_ego_frt_obj_exist_flg);
        fun("lc_ego_frt_obj_id", lc_ego_frt_obj_id);
        fun("lc_tar_frt_obj_exist_flg", lc_tar_frt_obj_exist_flg);
        fun("lc_tar_frt_obj_id", lc_tar_frt_obj_id);
        fun("lc_tar_bck_obj_exist_flg", lc_tar_bck_obj_exist_flg);
        fun("lc_tar_bck_obj_id", lc_tar_bck_obj_id);
        fun("lcp_cls", lcp_cls);
        fun("lcp_frt_passagegenflg", lcp_frt_passagegenflg);
        fun("lcp_bck_passagegenflg", lcp_bck_passagegenflg);
        fun("lcp_ego_frt_obj_exist_flg", lcp_ego_frt_obj_exist_flg);
        fun("lcp_ego_frt_obj_id", lcp_ego_frt_obj_id);
        fun("lcp_ego_bck_obj_exist_flg", lcp_ego_bck_obj_exist_flg);
        fun("lcp_ego_bck_obj_id", lcp_ego_bck_obj_id);
        fun("lcp_nr_obj_exist_flg", lcp_nr_obj_exist_flg);
        fun("lcp_nr_obj_id", lcp_nr_obj_id);
        fun("lcp_nr_frt_obj_exist_flg", lcp_nr_frt_obj_exist_flg);
        fun("lcp_nr_frt_obj_id", lcp_nr_frt_obj_id);
        fun("lcp_nr_bck_obj_exist_flg", lcp_nr_bck_obj_exist_flg);
        fun("lcp_nr_bck_obj_id", lcp_nr_bck_obj_id);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("passage_id", passage_id);
        fun("corridor_id", corridor_id);
        fun("longt_info", longt_info);
        fun("action_type", action_type);
        fun("feature_label", feature_label);
        fun("duration_time", duration_time);
        fun("interest_obj_id_list", interest_obj_id_list);
        fun("v_ego", v_ego);
        fun("lc_cls", lc_cls);
        fun("lc_passagegenflg", lc_passagegenflg);
        fun("lc_ego_frt_obj_exist_flg", lc_ego_frt_obj_exist_flg);
        fun("lc_ego_frt_obj_id", lc_ego_frt_obj_id);
        fun("lc_tar_frt_obj_exist_flg", lc_tar_frt_obj_exist_flg);
        fun("lc_tar_frt_obj_id", lc_tar_frt_obj_id);
        fun("lc_tar_bck_obj_exist_flg", lc_tar_bck_obj_exist_flg);
        fun("lc_tar_bck_obj_id", lc_tar_bck_obj_id);
        fun("lcp_cls", lcp_cls);
        fun("lcp_frt_passagegenflg", lcp_frt_passagegenflg);
        fun("lcp_bck_passagegenflg", lcp_bck_passagegenflg);
        fun("lcp_ego_frt_obj_exist_flg", lcp_ego_frt_obj_exist_flg);
        fun("lcp_ego_frt_obj_id", lcp_ego_frt_obj_id);
        fun("lcp_ego_bck_obj_exist_flg", lcp_ego_bck_obj_exist_flg);
        fun("lcp_ego_bck_obj_id", lcp_ego_bck_obj_id);
        fun("lcp_nr_obj_exist_flg", lcp_nr_obj_exist_flg);
        fun("lcp_nr_obj_id", lcp_nr_obj_id);
        fun("lcp_nr_frt_obj_exist_flg", lcp_nr_frt_obj_exist_flg);
        fun("lcp_nr_frt_obj_id", lcp_nr_frt_obj_id);
        fun("lcp_nr_bck_obj_exist_flg", lcp_nr_bck_obj_exist_flg);
        fun("lcp_nr_bck_obj_id", lcp_nr_bck_obj_id);
    }

    friend bool operator==(const ::dpc::Passage& lhs, const ::dpc::Passage& rhs) noexcept
    {
        return (lhs.passage_id == rhs.passage_id) && (lhs.corridor_id == rhs.corridor_id) && (lhs.longt_info == rhs.longt_info) && (lhs.action_type == rhs.action_type) && (lhs.feature_label == rhs.feature_label) && (fabs(static_cast<double>(lhs.duration_time - rhs.duration_time)) < DBL_EPSILON) && (lhs.interest_obj_id_list == rhs.interest_obj_id_list) && (fabs(static_cast<double>(lhs.v_ego - rhs.v_ego)) < DBL_EPSILON) && (lhs.lc_cls == rhs.lc_cls) && (lhs.lc_passagegenflg == rhs.lc_passagegenflg) && (lhs.lc_ego_frt_obj_exist_flg == rhs.lc_ego_frt_obj_exist_flg) && (lhs.lc_ego_frt_obj_id == rhs.lc_ego_frt_obj_id) && (lhs.lc_tar_frt_obj_exist_flg == rhs.lc_tar_frt_obj_exist_flg) && (lhs.lc_tar_frt_obj_id == rhs.lc_tar_frt_obj_id) && (lhs.lc_tar_bck_obj_exist_flg == rhs.lc_tar_bck_obj_exist_flg) && (lhs.lc_tar_bck_obj_id == rhs.lc_tar_bck_obj_id) && (lhs.lcp_cls == rhs.lcp_cls) && (lhs.lcp_frt_passagegenflg == rhs.lcp_frt_passagegenflg) && (lhs.lcp_bck_passagegenflg == rhs.lcp_bck_passagegenflg) && (lhs.lcp_ego_frt_obj_exist_flg == rhs.lcp_ego_frt_obj_exist_flg) && (lhs.lcp_ego_frt_obj_id == rhs.lcp_ego_frt_obj_id) && (lhs.lcp_ego_bck_obj_exist_flg == rhs.lcp_ego_bck_obj_exist_flg) && (lhs.lcp_ego_bck_obj_id == rhs.lcp_ego_bck_obj_id) && (lhs.lcp_nr_obj_exist_flg == rhs.lcp_nr_obj_exist_flg) && (lhs.lcp_nr_obj_id == rhs.lcp_nr_obj_id) && (lhs.lcp_nr_frt_obj_exist_flg == rhs.lcp_nr_frt_obj_exist_flg) && (lhs.lcp_nr_frt_obj_id == rhs.lcp_nr_frt_obj_id) && (lhs.lcp_nr_bck_obj_exist_flg == rhs.lcp_nr_bck_obj_exist_flg) && (lhs.lcp_nr_bck_obj_id == rhs.lcp_nr_bck_obj_id);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_PASSAGE_H
